from types import ModuleType
from typing import Any, Dict, Sequence, Type, Union

from django.core.cache.backends.base import BaseCache

class BaseMemcachedCache(BaseCache):
    def __init__(
        self,
        server: Union[str, Sequence[str]],
        params: Dict[str, Any],
        library: ModuleType,
        value_not_found_exception: Type[BaseException],
    ) -> None: ...

class MemcachedCache(BaseMemcachedCache):
    def __init__(self, server: Union[str, Sequence[str]], params: Dict[str, Any]) -> None: ...

class PyLibMCCache(BaseMemcachedCache):
    def __init__(self, server: Union[str, Sequence[str]], params: Dict[str, Any]) -> None: ...



from functools import wraps as wraps  # noqa: F401
from typing import Any, Callable, Generic, List, Optional, Sequence, Tuple, Type, TypeVar, Union, overload

from django.db.models.base import Model
from typing_extensions import Protocol, SupportsIndex

_T = TypeVar("_T")

class cached_property(Generic[_T]):
    func: Callable[..., _T] = ...
    name: str = ...
    def __init__(self, func: Callable[..., _T], name: str = ...) -> None: ...
    @overload
    def __get__(self, instance: None, cls: Type[Any] = ...) -> "cached_property[_T]": ...
    @overload
    def __get__(self, instance: object, cls: Type[Any] = ...) -> _T: ...

# Promise is only subclassed by a proxy class defined in the lazy function
# so it makes sense for it to have all the methods available in that proxy class
class Promise:
    def __init__(self, args: Any, kw: Any) -> None: ...
    def __reduce__(self) -> Tuple[Any, Tuple[Any]]: ...
    def __lt__(self, other: Any) -> bool: ...
    def __mod__(self, rhs: Any) -> Any: ...
    def __add__(self, other: Any) -> Any: ...
    def __radd__(self, other: Any) -> Any: ...
    def __deepcopy__(self: _T, memo: Any) -> _T: ...

class _StrPromise(Promise, Sequence[str]):
    def __add__(self, __s: str) -> str: ...
    # Incompatible with Sequence.__contains__
    def __contains__(self, __o: str) -> bool: ...  # type: ignore[override]
    def __ge__(self, __x: str) -> bool: ...
    def __getitem__(self, __i: SupportsIndex | slice) -> str: ...
    def __gt__(self, __x: str) -> bool: ...
    def __le__(self, __x: str) -> bool: ...
    # __len__ needed here because it defined abstract in Sequence[str]
    def __len__(self) -> int: ...
    def __lt__(self, __x: str) -> bool: ...
    def __mod__(self, __x: Any) -> str: ...
    def __mul__(self, __n: SupportsIndex) -> str: ...
    def __rmul__(self, __n: SupportsIndex) -> str: ...
    # Mypy requires this for the attribute hook to take effect
    def __getattribute__(self, __name: str) -> Any: ...

_StrOrPromise = Union[str, _StrPromise]
_C = TypeVar("_C", bound=Callable)

def lazy(func: _C, *resultclasses: Any) -> _C: ...
def lazystr(text: Any) -> _StrPromise: ...
def keep_lazy(*resultclasses: Any) -> Callable: ...
def keep_lazy_text(func: Callable) -> Callable: ...

empty: object

def new_method_proxy(func: Callable[..., _T]) -> Callable[..., _T]: ...

class LazyObject:
    def __init__(self) -> None: ...
    __getattr__: Callable = ...
    def __setattr__(self, name: str, value: Any) -> None: ...
    def __delattr__(self, name: str) -> None: ...
    def __reduce__(self) -> Tuple[Callable, Tuple[Model]]: ...
    def __copy__(self) -> LazyObject: ...
    __bytes__: Callable = ...
    __bool__: Callable = ...
    __dir__: Callable = ...
    __class__: Any = ...
    __ne__: Callable = ...
    __hash__: Callable = ...
    __getitem__: Callable = ...
    __setitem__: Callable = ...
    __delitem__: Callable = ...
    __iter__: Callable = ...
    __len__: Callable = ...
    __contains__: Callable = ...

def unpickle_lazyobject(wrapped: Model) -> Model: ...

class SimpleLazyObject(LazyObject):
    def __init__(self, func: Callable[[], Any]) -> None: ...
    def __copy__(self) -> SimpleLazyObject: ...

_PartitionMember = TypeVar("_PartitionMember")

def partition(
    predicate: Callable[[_PartitionMember], Union[int, bool]], values: List[_PartitionMember]
) -> Tuple[List[_PartitionMember], List[_PartitionMember]]: ...

_Get = TypeVar("_Get", covariant=True)
_Self = TypeVar("_Self")

class classproperty(Generic[_Get]):
    fget: Optional[Callable[[_Self], _Get]] = ...
    def __init__(self, method: Optional[Callable[[_Self], _Get]] = ...) -> None: ...
    def __get__(self, instance: Optional[_Self], cls: Type[_Self] = ...) -> _Get: ...
    def getter(self, method: Callable[[_Self], _Get]) -> classproperty[_Get]: ...

class _Getter(Protocol[_Get]):
    """Type fake to declare some read-only properties (until `property` builtin is generic)

    We can use something like `Union[_Getter[str], str]` in base class to avoid errors
    when redefining attribute with property or property with attribute.
    """

    @overload
    def __get__(self: _Self, __instance: None, __typeobj: Optional[Type[Any]]) -> _Self: ...
    @overload
    def __get__(self, __instance: Any, __typeobj: Optional[Type[Any]]) -> _Get: ...



from typing import Any, Callable, Dict, Iterable, List, Optional, Sequence, Tuple, Type, Union

from django.contrib.admin.filters import ListFilter
from django.contrib.admin.options import IS_POPUP_VAR as IS_POPUP_VAR  # noqa: F401
from django.contrib.admin.options import TO_FIELD_VAR as TO_FIELD_VAR
from django.contrib.admin.options import ModelAdmin, _DisplayT, _ListFilterT
from django.db.models.base import Model
from django.db.models.expressions import Expression
from django.db.models.options import Options
from django.db.models.query import QuerySet
from django.forms.formsets import BaseFormSet
from django.http.request import HttpRequest
from typing_extensions import Literal

ALL_VAR: str
ORDER_VAR: str
ORDER_TYPE_VAR: str
PAGE_VAR: str
SEARCH_VAR: str
ERROR_FLAG: str
IGNORED_PARAMS: Tuple[str, ...]

class ChangeList:
    model: Type[Model] = ...
    opts: Options = ...
    lookup_opts: Options = ...
    root_queryset: QuerySet = ...
    list_display: _DisplayT = ...
    list_display_links: _DisplayT = ...
    list_filter: Sequence[_ListFilterT] = ...
    date_hierarchy: Any = ...
    search_fields: Sequence[str] = ...
    list_select_related: Union[bool, Sequence[str]] = ...
    list_per_page: int = ...
    list_max_show_all: int = ...
    model_admin: ModelAdmin = ...
    preserved_filters: str = ...
    sortable_by: Optional[Sequence[str]] = ...
    page_num: int = ...
    show_all: bool = ...
    is_popup: bool = ...
    to_field: Any = ...
    params: Dict[str, Any] = ...
    list_editable: Sequence[str] = ...
    query: str = ...
    queryset: Any = ...
    title: str = ...
    pk_attname: str = ...
    formset: Optional[BaseFormSet]
    def __init__(
        self,
        request: HttpRequest,
        model: Type[Model],
        list_display: _DisplayT,
        list_display_links: _DisplayT,
        list_filter: Sequence[_ListFilterT],
        date_hierarchy: Optional[str],
        search_fields: Sequence[str],
        list_select_related: Union[bool, Sequence[str]],
        list_per_page: int,
        list_max_show_all: int,
        list_editable: Sequence[str],
        model_admin: ModelAdmin,
        sortable_by: Optional[Sequence[str]],
    ) -> None: ...
    def get_filters_params(self, params: Optional[Dict[str, Any]] = ...) -> Dict[str, Any]: ...
    def get_filters(
        self, request: HttpRequest
    ) -> Tuple[List[ListFilter], bool, Dict[str, Union[bool, str]], bool, bool]: ...
    def get_query_string(
        self, new_params: Optional[Dict[str, Any]] = ..., remove: Optional[Iterable[str]] = ...
    ) -> str: ...
    result_count: int = ...
    show_full_result_count: bool = ...
    show_admin_actions: bool = ...
    full_result_count: Optional[int] = ...
    result_list: Any = ...
    can_show_all: bool = ...
    multi_page: bool = ...
    paginator: Any = ...
    def get_results(self, request: HttpRequest) -> None: ...
    def get_ordering_field(self, field_name: Union[Callable, str]) -> Optional[Union[Expression, str]]: ...
    def get_ordering(self, request: HttpRequest, queryset: QuerySet) -> List[Union[Expression, str]]: ...
    def get_ordering_field_columns(self) -> Dict[int, Literal["desc", "asc"]]: ...
    def get_queryset(self, request: HttpRequest) -> QuerySet: ...
    filter_specs: List[ListFilter]
    has_filters: bool
    has_active_filters: bool
    clear_all_filters_qs: str
    def apply_select_related(self, qs: QuerySet) -> QuerySet: ...
    def has_related_field_in_list_display(self) -> bool: ...
    def url_for_result(self, result: Model) -> str: ...



from typing import Any, Callable, Dict, Iterable, List, Optional, Sequence, Tuple, Type, TypeVar, Union, overload
from uuid import UUID

from django.core import validators  # due to weird mypy.stubtest error
from django.db import models
from django.db.models.base import Model
from django.db.models.expressions import Combinable
from django.db.models.fields import Field, _AllLimitChoicesTo, _ErrorMessagesT, _FieldChoices, _LimitChoicesTo
from django.db.models.fields.mixins import FieldCacheMixin
from django.db.models.fields.related_descriptors import ForwardManyToOneDescriptor as ForwardManyToOneDescriptor
from django.db.models.fields.related_descriptors import (  # noqa: F401
    ForwardOneToOneDescriptor as ForwardOneToOneDescriptor,
)
from django.db.models.fields.related_descriptors import ManyToManyDescriptor as ManyToManyDescriptor
from django.db.models.fields.related_descriptors import ReverseManyToOneDescriptor as ReverseManyToOneDescriptor
from django.db.models.fields.related_descriptors import ReverseOneToOneDescriptor as ReverseOneToOneDescriptor
from django.db.models.fields.reverse_related import ForeignObjectRel as ForeignObjectRel  # noqa: F401
from django.db.models.fields.reverse_related import ManyToManyRel as ManyToManyRel
from django.db.models.fields.reverse_related import ManyToOneRel as ManyToOneRel
from django.db.models.fields.reverse_related import OneToOneRel as OneToOneRel
from django.db.models.manager import RelatedManager
from django.db.models.query_utils import FilteredRelation, PathInfo, Q
from django.utils.functional import _StrOrPromise
from typing_extensions import Literal

_T = TypeVar("_T", bound=models.Model)
_F = TypeVar("_F", bound=models.Field)

RECURSIVE_RELATIONSHIP_CONSTANT: Literal["self"] = ...

def resolve_relation(scope_model: Type[Model], relation: Union[str, Type[Model]]) -> Union[str, Type[Model]]: ...

# __set__ value type
_ST = TypeVar("_ST")
# __get__ return type
_GT = TypeVar("_GT")

class RelatedField(FieldCacheMixin, Field[_ST, _GT]):
    one_to_many: bool = ...
    one_to_one: bool = ...
    many_to_many: bool = ...
    many_to_one: bool = ...
    opts: Any = ...

    remote_field: ForeignObjectRel
    rel_class: Type[ForeignObjectRel]
    swappable: bool
    @property
    def related_model(self) -> Type[Model]: ...  # type: ignore
    def get_forward_related_filter(self, obj: Model) -> Dict[str, Union[int, UUID]]: ...
    def get_reverse_related_filter(self, obj: Model) -> Q: ...
    @property
    def swappable_setting(self) -> Optional[str]: ...
    def set_attributes_from_rel(self) -> None: ...
    def do_related_class(self, other: Type[Model], cls: Type[Model]) -> None: ...
    def get_limit_choices_to(self) -> _LimitChoicesTo: ...
    def related_query_name(self) -> str: ...
    @property
    def target_field(self) -> Field: ...

class ForeignObject(RelatedField[_ST, _GT]):
    remote_field: ForeignObjectRel
    rel_class: Type[ForeignObjectRel]
    from_fields: Sequence[str]
    to_fields: Sequence[str | None]  # None occurs in ForeignKey, where to_field defaults to None
    swappable: bool
    def __init__(
        self,
        to: Union[Type[Model], str],
        on_delete: Callable[..., None],
        from_fields: Sequence[str],
        to_fields: Sequence[str],
        rel: Optional[ForeignObjectRel] = ...,
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[_AllLimitChoicesTo] = ...,
        parent_link: bool = ...,
        swappable: bool = ...,
        *,
        db_constraint: bool = ...,
        verbose_name: Optional[_StrOrPromise] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: _StrOrPromise = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesT] = ...,
    ) -> None: ...
    def resolve_related_fields(self) -> List[Tuple[Field, Field]]: ...
    @property
    def related_fields(self) -> List[Tuple[Field, Field]]: ...
    @property
    def reverse_related_fields(self) -> List[Tuple[Field, Field]]: ...
    @property
    def local_related_fields(self) -> Tuple[Field, ...]: ...
    @property
    def foreign_related_fields(self) -> Tuple[Field, ...]: ...

class ForeignKey(ForeignObject[_ST, _GT]):
    _pyi_private_set_type: Union[Any, Combinable]
    _pyi_private_get_type: Any

    remote_field: ManyToOneRel
    rel_class: Type[ManyToOneRel]
    def __init__(
        self,
        to: Union[Type[Model], str],
        on_delete: Callable[..., None],
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[_AllLimitChoicesTo] = ...,
        parent_link: bool = ...,
        to_field: Optional[str] = ...,
        db_constraint: bool = ...,
        *,
        verbose_name: Optional[_StrOrPromise] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: _StrOrPromise = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesT] = ...,
    ) -> None: ...
    # class access
    @overload  # type: ignore
    def __get__(self, instance: None, owner: Any) -> ForwardManyToOneDescriptor: ...
    # Model instance access
    @overload
    def __get__(self, instance: Model, owner: Any) -> _GT: ...
    # non-Model instances
    @overload
    def __get__(self: _F, instance: Any, owner: Any) -> _F: ...

class OneToOneField(ForeignKey[_ST, _GT]):
    _pyi_private_set_type: Union[Any, Combinable]
    _pyi_private_get_type: Any

    remote_field: OneToOneRel
    rel_class: Type[OneToOneRel]
    def __init__(
        self,
        to: Union[Type[Model], str],
        on_delete: Any,
        to_field: Optional[str] = ...,
        *,
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[_AllLimitChoicesTo] = ...,
        parent_link: bool = ...,
        db_constraint: bool = ...,
        verbose_name: Optional[_StrOrPromise] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: _StrOrPromise = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesT] = ...,
    ) -> None: ...
    # class access
    @overload  # type: ignore
    def __get__(self, instance: None, owner: Any) -> ForwardOneToOneDescriptor: ...
    # Model instance access
    @overload
    def __get__(self, instance: Model, owner: Any) -> _GT: ...
    # non-Model instances
    @overload
    def __get__(self: _F, instance: Any, owner: Any) -> _F: ...

class ManyToManyField(RelatedField[_ST, _GT]):
    _pyi_private_set_type: Sequence[Any]
    _pyi_private_get_type: RelatedManager[Any]

    description: str = ...
    has_null_arg: bool = ...
    swappable: bool = ...

    many_to_many: Literal[True]
    many_to_one: Literal[False]
    one_to_many: Literal[False]
    one_to_one: Literal[False]

    remote_field: ManyToManyRel
    rel_class: Type[ManyToManyRel]
    def __init__(
        self,
        to: Union[Type[Model], str],
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[_AllLimitChoicesTo] = ...,
        symmetrical: Optional[bool] = ...,
        through: Union[str, Type[Model], None] = ...,
        through_fields: Optional[Tuple[str, str]] = ...,
        db_constraint: bool = ...,
        db_table: Optional[str] = ...,
        swappable: bool = ...,
        *,
        verbose_name: Optional[_StrOrPromise] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: _StrOrPromise = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesT] = ...,
    ) -> None: ...
    # class access
    @overload  # type: ignore
    def __get__(self, instance: None, owner: Any) -> ManyToManyDescriptor: ...
    # Model instance access
    @overload
    def __get__(self, instance: Model, owner: Any) -> _GT: ...
    # non-Model instances
    @overload
    def __get__(self: _F, instance: Any, owner: Any) -> _F: ...
    def get_path_info(self, filtered_relation: Optional[FilteredRelation] = ...) -> List[PathInfo]: ...
    def get_reverse_path_info(self, filtered_relation: Optional[FilteredRelation] = ...) -> List[PathInfo]: ...
    def contribute_to_related_class(self, cls: Type[Model], related: RelatedField) -> None: ...
    def m2m_db_table(self) -> str: ...
    def m2m_column_name(self) -> str: ...
    def m2m_reverse_name(self) -> str: ...
    def m2m_reverse_field_name(self) -> str: ...
    def m2m_target_field_name(self) -> str: ...
    def m2m_reverse_target_field_name(self) -> str: ...

def create_many_to_many_intermediary_model(field: ManyToManyField, klass: Type[Model]) -> Type[Model]: ...



from typing import Any, Dict, Optional, Sequence, Type, Union

from django import forms as forms
from django.contrib.postgres.validators import ArrayMaxLengthValidator as ArrayMaxLengthValidator
from django.contrib.postgres.validators import ArrayMinLengthValidator as ArrayMinLengthValidator
from django.core.exceptions import ValidationError as ValidationError
from django.db.models.fields import _ErrorMessagesT
from django.forms.fields import _ClassLevelWidgetT
from django.forms.utils import _DataT, _FilesT
from django.forms.widgets import Media, _OptAttrs

from ..utils import prefix_validation_error as prefix_validation_error

class SimpleArrayField(forms.CharField):
    default_error_messages: _ErrorMessagesT = ...
    base_field: forms.Field
    delimiter: str
    min_length: Optional[int]
    max_length: Optional[int]
    def __init__(
        self,
        base_field: forms.Field,
        *,
        delimiter: str = ...,
        max_length: Optional[int] = ...,
        min_length: Optional[int] = ...,
        **kwargs: Any
    ) -> None: ...
    def clean(self, value: Any) -> Sequence[Any]: ...
    def prepare_value(self, value: Any) -> Any: ...
    def to_python(self, value: Any) -> Sequence[Any]: ...  # type: ignore
    def validate(self, value: Sequence[Any]) -> None: ...
    def run_validators(self, value: Sequence[Any]) -> None: ...
    def has_changed(self, initial: Any, data: Any) -> bool: ...

class SplitArrayWidget(forms.Widget):
    template_name: str
    widget: _ClassLevelWidgetT
    size: int
    def __init__(self, widget: Union[forms.Widget, Type[forms.Widget]], size: int, **kwargs: Any) -> None: ...
    @property
    def is_hidden(self) -> bool: ...
    def value_from_datadict(self, data: _DataT, files: _FilesT, name: str) -> Any: ...
    def value_omitted_from_data(self, data: _DataT, files: _FilesT, name: str) -> bool: ...
    def id_for_label(self, id_: str) -> str: ...
    def get_context(self, name: str, value: Any, attrs: Optional[_OptAttrs] = ...) -> Dict[str, Any]: ...
    @property
    def media(self) -> Media: ...  # type: ignore
    @property
    def needs_multipart_form(self) -> bool: ...  # type: ignore

class SplitArrayField(forms.Field):
    default_error_messages: _ErrorMessagesT = ...
    base_field: forms.Field
    size: int
    remove_trailing_nulls: bool
    def __init__(
        self, base_field: forms.Field, size: int, *, remove_trailing_nulls: bool = ..., **kwargs: Any
    ) -> None: ...
    def to_python(self, value: Any) -> Sequence[Any]: ...
    def clean(self, value: Any) -> Sequence[Any]: ...
    def has_changed(self, initial: Any, data: Any) -> bool: ...



from typing import List, Optional, Sequence, Set, Tuple, Union

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.migrations.migration import Migration
from typing_extensions import Protocol

from .loader import MigrationLoader
from .recorder import MigrationRecorder
from .state import ProjectState

class _ProgressCallbackT(Protocol):
    def __call__(self, __action: str, __migration: Optional[Migration] = ..., __fake: Optional[bool] = ...) -> None: ...

class MigrationExecutor:
    connection: BaseDatabaseWrapper = ...
    loader: MigrationLoader = ...
    recorder: MigrationRecorder = ...
    progress_callback: Optional[_ProgressCallbackT] = ...
    def __init__(
        self,
        connection: Optional[BaseDatabaseWrapper],
        progress_callback: Optional[_ProgressCallbackT] = ...,
    ) -> None: ...
    def migration_plan(
        self, targets: Union[Sequence[Tuple[str, Optional[str]]], Set[Tuple[str, str]]], clean_start: bool = ...
    ) -> List[Tuple[Migration, bool]]: ...
    def migrate(
        self,
        targets: Optional[Sequence[Tuple[str, Optional[str]]]],
        plan: Optional[Sequence[Tuple[Migration, bool]]] = ...,
        state: Optional[ProjectState] = ...,
        fake: bool = ...,
        fake_initial: bool = ...,
    ) -> ProjectState: ...
    def apply_migration(
        self, state: ProjectState, migration: Migration, fake: bool = ..., fake_initial: bool = ...
    ) -> ProjectState: ...
    def record_migration(self, migration: Migration) -> None: ...
    def unapply_migration(self, state: ProjectState, migration: Migration, fake: bool = ...) -> ProjectState: ...
    def check_replacements(self) -> None: ...
    def detect_soft_applied(
        self, project_state: Optional[ProjectState], migration: Migration
    ) -> Tuple[bool, ProjectState]: ...



from datetime import date
from datetime import datetime as builtin_datetime
from datetime import time, timedelta
from typing import Optional, Pattern

date_re: Pattern[str]
time_re: Pattern[str]
datetime_re: Pattern[str]
standard_duration_re: Pattern[str]
iso8601_duration_re: Pattern[str]
postgres_interval_re: Pattern[str]

def parse_date(value: str) -> Optional[date]: ...
def parse_time(value: str) -> Optional[time]: ...
def parse_datetime(value: str) -> Optional[builtin_datetime]: ...
def parse_duration(value: str) -> Optional[timedelta]: ...



import os
from pathlib import Path
from typing import Union

_PathCompatible = Union[str, os.PathLike[str]]

def safe_join(base: _PathCompatible, *paths: _PathCompatible) -> str: ...
def symlinks_supported() -> bool: ...
def to_path(value: Union[Path, str]) -> Path: ...



from argparse import ArgumentParser, HelpFormatter, Namespace
from io import TextIOBase
from typing import Any, Callable, Iterable, List, Optional, Sequence, Set, TextIO, Tuple, Union

from django.apps.config import AppConfig
from django.core.management.color import Style
from django.utils.datastructures import _ListOrTuple
from typing_extensions import Literal

class CommandError(Exception):
    def __init__(self, *args: Any, returncode: int = ..., **kwargs: Any) -> None: ...

class SystemCheckError(CommandError): ...

class CommandParser(ArgumentParser):
    missing_args_message: Optional[str] = ...
    called_from_command_line: Optional[bool] = ...
    def __init__(
        self,
        *,
        missing_args_message: Optional[str] = ...,
        called_from_command_line: Optional[bool] = ...,
        **kwargs: Any
    ) -> None: ...
    def error(self, message: str) -> Any: ...

def handle_default_options(options: Namespace) -> None: ...
def no_translations(handle_func: Callable) -> Callable: ...

class DjangoHelpFormatter(HelpFormatter):
    show_last: Set[str] = ...
    def add_usage(self, usage: Optional[str], actions: Iterable[Any], *args: Any, **kwargs: Any) -> Any: ...
    def add_arguments(self, actions: Any) -> Any: ...

class OutputWrapper(TextIOBase):
    @property
    def style_func(self) -> Callable[[str], str]: ...
    @style_func.setter
    def style_func(self, style_func: Optional[Callable[[str], str]]) -> None: ...
    ending: str = ...
    def __init__(self, out: TextIO, ending: str = ...) -> None: ...
    def __getattr__(self, name: str) -> Callable: ...
    def flush(self) -> None: ...
    def isatty(self) -> bool: ...
    def write(  # type: ignore[override]
        self, msg: str = ..., style_func: Optional[Callable[[str], str]] = ..., ending: Optional[str] = ...
    ) -> None: ...

class BaseCommand:
    help: str = ...
    output_transaction: bool = ...
    requires_migrations_checks: bool = ...
    requires_system_checks: Union[_ListOrTuple[str], Literal["__all__"]] = ...
    base_stealth_options: Tuple[str, ...] = ...
    stealth_options: Tuple[str, ...] = ...
    stdout: OutputWrapper = ...
    stderr: OutputWrapper = ...
    style: Style = ...
    def __init__(
        self,
        stdout: Optional[TextIO] = ...,
        stderr: Optional[TextIO] = ...,
        no_color: bool = ...,
        force_color: bool = ...,
    ) -> None: ...
    def get_version(self) -> str: ...
    def create_parser(self, prog_name: str, subcommand: str, **kwargs: Any) -> CommandParser: ...
    def add_arguments(self, parser: CommandParser) -> None: ...
    def print_help(self, prog_name: str, subcommand: str) -> None: ...
    def run_from_argv(self, argv: List[str]) -> None: ...
    def execute(self, *args: Any, **options: Any) -> Optional[str]: ...
    def check(
        self,
        app_configs: Optional[Sequence[AppConfig]] = ...,
        tags: Optional[Sequence[str]] = ...,
        display_num_errors: bool = ...,
        include_deployment_checks: bool = ...,
        fail_level: int = ...,
        databases: Optional[Sequence[str]] = ...,
    ) -> None: ...
    def check_migrations(self) -> None: ...
    def handle(self, *args: Any, **options: Any) -> Optional[str]: ...

class AppCommand(BaseCommand):
    missing_args_message: str = ...
    def add_arguments(self, parser: CommandParser) -> None: ...
    def handle(self, *app_labels: str, **options: Any) -> Optional[str]: ...
    def handle_app_config(self, app_config: Any, **options: Any) -> Optional[str]: ...

class LabelCommand(BaseCommand):
    label: str = ...
    missing_args_message: Any = ...
    def add_arguments(self, parser: CommandParser) -> None: ...
    def handle(self, *labels: str, **options: Any) -> Optional[str]: ...
    def handle_label(self, label: str, **options: Any) -> Optional[str]: ...



from typing import Any, Dict, Optional, Type, Union

from django import forms
from django.db.models.fields import _ErrorMessagesT
from django.forms.fields import _ClassLevelWidgetT

class HStoreField(forms.CharField):
    widget: _ClassLevelWidgetT
    default_error_messages: _ErrorMessagesT = ...
    def prepare_value(self, value: Any) -> Any: ...
    def to_python(self, value: Any) -> Dict[str, Optional[str]]: ...  # type: ignore
    def has_changed(self, initial: Any, data: Any) -> bool: ...



from typing import Callable, Dict, Optional, Type, TypeVar, Union

from django.contrib.sitemaps import GenericSitemap, Sitemap
from django.http.request import HttpRequest
from django.template.response import TemplateResponse

_C = TypeVar("_C", bound=Callable)

def x_robots_tag(func: _C) -> _C: ...
def index(
    request: HttpRequest,
    sitemaps: Dict[str, Union[Type[Sitemap], Sitemap]],
    template_name: str = ...,
    content_type: str = ...,
    sitemap_url_name: str = ...,
) -> TemplateResponse: ...
def sitemap(
    request: HttpRequest,
    sitemaps: Dict[str, Union[Type[Sitemap], Sitemap]],
    section: Optional[str] = ...,
    template_name: str = ...,
    content_type: str = ...,
) -> TemplateResponse: ...



from typing import Dict, Optional, Type

from django.contrib.gis.feeds import Feed
from django.http import HttpRequest, HttpResponse

def feed(request: HttpRequest, url: str, feed_dict: Optional[Dict[str, Type[Feed]]] = ...) -> HttpResponse: ...



from typing import Any, Dict, Iterable, Optional, Sequence, Tuple

from django.db.backends.base.base import BaseDatabaseWrapper

class BaseDatabaseClient:
    executable_name: Optional[str] = ...
    connection: BaseDatabaseWrapper
    def __init__(self, connection: BaseDatabaseWrapper) -> None: ...
    @classmethod
    def settings_to_cmd_args_env(
        cls,
        settings_dict: Dict[str, Any],
        parameters: Iterable[str],
    ) -> Tuple[Sequence[str], Optional[Dict[str, str]]]: ...
    def runshell(self, parameters: Iterable[str]) -> None: ...



from typing import Any, Dict, Optional, Type

from django.http import HttpRequest
from django.template.backends.base import BaseEngine
from django.template.backends.django import DjangoTemplates as DjangoTemplatesR
from django.template.backends.jinja2 import Jinja2 as Jinja2R
from django.template.base import Template

def get_default_renderer() -> BaseRenderer: ...

class BaseRenderer:
    def get_template(self, template_name: str) -> Any: ...
    def render(self, template_name: str, context: Dict[str, Any], request: Optional[HttpRequest] = ...) -> str: ...

class EngineMixin:
    def get_template(self, template_name: str) -> Any: ...
    @property
    def engine(self) -> BaseEngine: ...

class DjangoTemplates(EngineMixin, BaseRenderer):
    backend: Type[DjangoTemplatesR] = ...

class Jinja2(EngineMixin, BaseRenderer):
    @property
    def backend(self) -> Type[Jinja2R]: ...

class TemplatesSetting(BaseRenderer):
    def get_template(self, template_name: str) -> Optional[Template]: ...



from typing import Any, Dict, Iterable, List, Mapping, Optional, Sequence, Tuple, Union

from django import forms
from django.contrib.admin.sites import AdminSite
from django.core.files.base import File
from django.db.models.fields import _FieldChoices
from django.db.models.fields.reverse_related import ForeignObjectRel, ManyToManyRel, ManyToOneRel
from django.forms.models import ModelChoiceIterator
from django.forms.widgets import Media, _OptAttrs
from django.utils.functional import _StrOrPromise

class FilteredSelectMultiple(forms.SelectMultiple):
    verbose_name: _StrOrPromise = ...
    is_stacked: bool = ...
    def __init__(
        self,
        verbose_name: _StrOrPromise,
        is_stacked: bool,
        attrs: Optional[_OptAttrs] = ...,
        choices: _FieldChoices = ...,
    ) -> None: ...

class AdminDateWidget(forms.DateInput):
    def __init__(self, attrs: Optional[_OptAttrs] = ..., format: Optional[str] = ...) -> None: ...

class AdminTimeWidget(forms.TimeInput):
    def __init__(self, attrs: Optional[_OptAttrs] = ..., format: Optional[str] = ...) -> None: ...

class AdminSplitDateTime(forms.SplitDateTimeWidget):
    template_name: str
    def __init__(self, attrs: Optional[_OptAttrs] = ...) -> None: ...
    def get_context(self, name: str, value: Any, attrs: Optional[_OptAttrs]) -> Dict[str, Any]: ...

class AdminRadioSelect(forms.RadioSelect): ...
class AdminFileWidget(forms.ClearableFileInput): ...

def url_params_from_lookup_dict(lookups: Any) -> Dict[str, str]: ...

class ForeignKeyRawIdWidget(forms.TextInput):
    rel: ManyToOneRel = ...
    admin_site: AdminSite = ...
    db: Optional[str] = ...
    def __init__(
        self, rel: ManyToOneRel, admin_site: AdminSite, attrs: Optional[_OptAttrs] = ..., using: Optional[str] = ...
    ) -> None: ...
    def base_url_parameters(self) -> Dict[str, str]: ...
    def get_context(self, name: str, value: Any, attrs: Optional[_OptAttrs]) -> Dict[str, Any]: ...
    def url_parameters(self) -> Dict[str, str]: ...
    def label_and_url_for_value(self, value: Any) -> Tuple[str, str]: ...

class ManyToManyRawIdWidget(ForeignKeyRawIdWidget):
    rel: ManyToManyRel = ...  # type: ignore
    def get_context(self, name: str, value: Any, attrs: Optional[_OptAttrs]) -> Dict[str, Any]: ...
    def url_parameters(self) -> Dict[str, str]: ...
    def label_and_url_for_value(self, value: Any) -> Tuple[str, str]: ...
    def format_value(self, value: Any) -> Optional[str]: ...
    def value_from_datadict(self, data: Mapping[str, Any], files: Mapping[str, Iterable[File]], name: str) -> Any: ...

class RelatedFieldWidgetWrapper(forms.Widget):
    template_name: str = ...
    choices: ModelChoiceIterator = ...
    widget: forms.ChoiceWidget = ...
    rel: ManyToOneRel = ...
    can_add_related: bool = ...
    can_change_related: bool = ...
    can_delete_related: bool = ...
    can_view_related: bool = ...
    admin_site: AdminSite = ...
    def __init__(
        self,
        widget: forms.ChoiceWidget,
        rel: ManyToOneRel,
        admin_site: AdminSite,
        can_add_related: Optional[bool] = ...,
        can_change_related: bool = ...,
        can_delete_related: bool = ...,
        can_view_related: bool = ...,
    ) -> None: ...
    @property
    def media(self) -> Media: ...  # type: ignore
    @property
    def is_hidden(self) -> bool: ...
    def get_related_url(self, info: Tuple[str, str], action: str, *args: Any) -> str: ...
    def get_context(self, name: str, value: Any, attrs: Optional[_OptAttrs]) -> Dict[str, Any]: ...
    def value_from_datadict(self, data: Mapping[str, Any], files: Mapping[str, Iterable[File]], name: str) -> Any: ...
    def value_omitted_from_data(
        self, data: Mapping[str, Any], files: Mapping[str, Iterable[File]], name: str
    ) -> bool: ...
    def id_for_label(self, id_: str) -> str: ...

class AdminTextareaWidget(forms.Textarea):
    def __init__(self, attrs: Optional[_OptAttrs] = ...) -> None: ...

class AdminTextInputWidget(forms.TextInput):
    def __init__(self, attrs: Optional[_OptAttrs] = ...) -> None: ...

class AdminEmailInputWidget(forms.EmailInput):
    def __init__(self, attrs: Optional[_OptAttrs] = ...) -> None: ...

class AdminURLFieldWidget(forms.URLInput):
    template_name: str
    def __init__(self, attrs: Optional[_OptAttrs] = ..., validator_class: Any = ...) -> None: ...
    def get_context(self, name: str, value: Any, attrs: Optional[_OptAttrs]) -> Dict[str, Any]: ...

class AdminIntegerFieldWidget(forms.NumberInput):
    def __init__(self, attrs: Optional[_OptAttrs] = ...) -> None: ...
    class_name: str = ...

class AdminBigIntegerFieldWidget(AdminIntegerFieldWidget):
    class_name: str = ...

class AdminUUIDInputWidget(forms.TextInput):
    def __init__(self, attrs: Optional[_OptAttrs] = ...) -> None: ...

SELECT2_TRANSLATIONS: Dict[str, str] = ...

class AutocompleteMixin:
    url_name: str = ...
    field: Any = ...
    admin_site: AdminSite = ...
    db: Optional[str] = ...
    choices: Any = ...
    attrs: _OptAttrs = ...
    def __init__(
        self,
        field: Any,
        admin_site: AdminSite,
        attrs: Optional[_OptAttrs] = ...,
        choices: Any = ...,
        using: Optional[str] = ...,
    ) -> None: ...
    def get_url(self) -> str: ...
    @property
    def media(self) -> Media: ...
    def build_attrs(self, base_attrs: _OptAttrs, extra_attrs: Optional[_OptAttrs] = ...) -> Dict[str, Any]: ...
    # typo in source: `attr` instead of `attrs`
    def optgroups(
        self, name: str, value: Sequence[str], attr: Optional[_OptAttrs] = ...
    ) -> List[Tuple[Optional[str], List[Dict[str, Any]], Optional[int]]]: ...

class AutocompleteSelect(AutocompleteMixin, forms.Select): ...  # type: ignore
class AutocompleteSelectMultiple(AutocompleteMixin, forms.SelectMultiple): ...  # type: ignore



from typing import (
    Any,
    Callable,
    Collection,
    Dict,
    Iterable,
    Iterator,
    List,
    Optional,
    Sequence,
    Set,
    Tuple,
    Type,
    Union,
)

from django.db import IntegrityError
from django.db.models.base import Model
from django.db.models.fields import Field
from django.db.models.options import Options
from django.db.models.query import QuerySet
from django.utils.datastructures import _IndexableCollection

def CASCADE(
    collector: "Collector",
    field: Field[Any, Any],
    sub_objs: QuerySet[Model],
    using: str,
) -> None: ...
def SET_NULL(
    collector: "Collector",
    field: Field[Any, Any],
    sub_objs: QuerySet[Model],
    using: str,
) -> None: ...
def SET_DEFAULT(
    collector: "Collector",
    field: Field[Any, Any],
    sub_objs: QuerySet[Model],
    using: str,
) -> None: ...
def DO_NOTHING(
    collector: "Collector",
    field: Field[Any, Any],
    sub_objs: QuerySet[Model],
    using: str,
) -> None: ...
def PROTECT(
    collector: "Collector",
    field: Field[Any, Any],
    sub_objs: QuerySet[Model],
    using: str,
) -> None: ...
def RESTRICT(
    collector: "Collector",
    field: Field[Any, Any],
    sub_objs: QuerySet[Model],
    using: str,
) -> None: ...
def SET(value: Any) -> Callable[..., Any]: ...
def get_candidate_relations_to_delete(opts: Options) -> Iterable[Field]: ...

class ProtectedError(IntegrityError):
    protected_objects: Set[Model]
    def __init__(self, msg: str, protected_objects: Set[Model]) -> None: ...

class RestrictedError(IntegrityError):
    restricted_objects: Set[Model]
    def __init__(self, msg: str, restricted_objects: Set[Model]) -> None: ...

class Collector:
    using: str
    data: Any
    field_updates: Any
    restricted_objects: Any
    fast_deletes: Any
    dependencies: Any
    def __init__(self, using: str) -> None: ...
    def add(
        self,
        objs: _IndexableCollection[Model],
        source: Optional[Type[Model]] = ...,
        nullable: bool = ...,
        reverse_dependency: bool = ...,
    ) -> List[Model]: ...
    def add_dependency(self, model: Type[Model], dependency: Type[Model], reverse_dependency: bool = ...) -> None: ...
    def add_field_update(self, field: Field, value: Any, objs: _IndexableCollection[Model]) -> None: ...
    def add_restricted_objects(self, field: Field, objs: _IndexableCollection[Model]) -> None: ...
    def clear_restricted_objects_from_set(self, model: Type[Model], objs: Set[Model]) -> None: ...
    def clear_restricted_objects_from_queryset(self, model: Type[Model], qs: QuerySet[Model]) -> None: ...
    def can_fast_delete(self, objs: Union[Model, Iterable[Model]], from_field: Optional[Field] = ...) -> bool: ...
    def get_del_batches(
        self, objs: _IndexableCollection[Model], fields: Iterable[Field]
    ) -> Sequence[Sequence[Model]]: ...
    def collect(
        self,
        objs: _IndexableCollection[Optional[Model]],
        source: Optional[Type[Model]] = ...,
        nullable: bool = ...,
        collect_related: bool = ...,
        source_attr: Optional[str] = ...,
        reverse_dependency: bool = ...,
        keep_parents: bool = ...,
        fail_on_restricted: bool = ...,
    ) -> None: ...
    def related_objects(
        self, related_model: Type[Model], related_fields: Iterable[Field], objs: _IndexableCollection[Model]
    ) -> QuerySet[Model]: ...
    def instances_with_model(self) -> Iterator[Tuple[Type[Model], Model]]: ...
    def sort(self) -> None: ...
    def delete(self) -> Tuple[int, Dict[str, int]]: ...



from typing import Any, Optional, Tuple, Type, Union

from django import forms
from django.db.models.fields import _ErrorMessagesT
from django.forms.widgets import MultiWidget, _OptAttrs
from psycopg2.extras import Range

class RangeWidget(MultiWidget):
    def __init__(
        self, base_widget: Union[forms.Widget, Type[forms.Widget]], attrs: Optional[_OptAttrs] = ...
    ) -> None: ...
    def decompress(self, value: Any) -> Tuple[Optional[Any], Optional[Any]]: ...

class HiddenRangeWidget(RangeWidget):
    def __init__(self, attrs: Optional[_OptAttrs] = ...) -> None: ...

class BaseRangeField(forms.MultiValueField):
    default_error_messages: _ErrorMessagesT
    base_field: Type[forms.Field]
    range_type: Type[Range]
    hidden_widget: Type[forms.Widget]
    def __init__(self, **kwargs: Any) -> None: ...
    def prepare_value(self, value: Any) -> Any: ...
    def compress(self, values: Tuple[Optional[Any], Optional[Any]]) -> Optional[Range]: ...

class IntegerRangeField(BaseRangeField):
    default_error_messages: _ErrorMessagesT
    base_field: Type[forms.Field]
    range_type: Type[Range]

class DecimalRangeField(BaseRangeField):
    default_error_messages: _ErrorMessagesT
    base_field: Type[forms.Field]
    range_type: Type[Range]

class DateTimeRangeField(BaseRangeField):
    default_error_messages: _ErrorMessagesT
    base_field: Type[forms.Field]
    range_type: Type[Range]

class DateRangeField(BaseRangeField):
    default_error_messages: _ErrorMessagesT
    base_field: Type[forms.Field]
    range_type: Type[Range]



import datetime
from contextlib import contextmanager
from decimal import Decimal
from logging import Logger
from types import TracebackType
from typing import (
    Any,
    Dict,
    Generator,
    Iterator,
    List,
    Mapping,
    Optional,
    Protocol,
    Sequence,
    Tuple,
    Type,
    Union,
    overload,
)
from uuid import UUID

from typing_extensions import Literal

logger: Logger

# Protocol matching psycopg2.sql.Composable, to avoid depending psycopg2
class _Composable(Protocol):
    def as_string(self, context: Any) -> str: ...
    def __add__(self, other: _Composable) -> _Composable: ...
    def __mul__(self, n: int) -> _Composable: ...

_ExecuteQuery = Union[str, _Composable]

# Python types that can be adapted to SQL.
_SQLType = Union[
    None, bool, int, float, Decimal, str, bytes, datetime.date, datetime.datetime, UUID, Tuple[Any, ...], List[Any]
]
_ExecuteParameters = Optional[Union[Sequence[_SQLType], Mapping[str, _SQLType]]]

class CursorWrapper:
    cursor: Any = ...
    db: Any = ...
    def __init__(self, cursor: Any, db: Any) -> None: ...
    WRAP_ERROR_ATTRS: Any = ...
    def __getattr__(self, attr: str) -> Any: ...
    def __iter__(self) -> Iterator[Tuple[Any, ...]]: ...
    def __enter__(self) -> CursorWrapper: ...
    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_value: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None: ...
    def callproc(
        self, procname: str, params: Optional[Sequence[Any]] = ..., kparams: Optional[Dict[str, int]] = ...
    ) -> Any: ...
    def execute(self, sql: _ExecuteQuery, params: _ExecuteParameters = ...) -> Any: ...
    def executemany(self, sql: _ExecuteQuery, param_list: Sequence[_ExecuteParameters]) -> Any: ...

class CursorDebugWrapper(CursorWrapper):
    cursor: Any
    db: Any
    @contextmanager
    def debug_sql(
        self,
        sql: Optional[str] = ...,
        params: Optional[Union[_ExecuteParameters, Sequence[_ExecuteParameters]]] = ...,
        use_last_executed_query: bool = ...,
        many: bool = ...,
    ) -> Generator[None, None, None]: ...

@overload
def typecast_date(s: Union[None, Literal[""]]) -> None: ...  # type: ignore
@overload
def typecast_date(s: str) -> datetime.date: ...
@overload
def typecast_time(s: Union[None, Literal[""]]) -> None: ...  # type: ignore
@overload
def typecast_time(s: str) -> datetime.time: ...
@overload
def typecast_timestamp(s: Union[None, Literal[""]]) -> None: ...  # type: ignore
@overload
def typecast_timestamp(s: str) -> datetime.datetime: ...
def split_identifier(identifier: str) -> Tuple[str, str]: ...
def truncate_name(identifier: str, length: Optional[int] = ..., hash_len: int = ...) -> str: ...
def format_number(
    value: Optional[Decimal], max_digits: Optional[int], decimal_places: Optional[int]
) -> Optional[str]: ...
def strip_quotes(table_name: str) -> str: ...



from logging import Logger
from typing import Any

from django.db.backends.mysql.schema import DatabaseSchemaEditor as DatabaseSchemaEditor

logger: Logger

class MySQLGISSchemaEditor(DatabaseSchemaEditor):
    sql_add_spatial_index: str = ...
    sql_drop_spatial_index: str = ...
    geometry_sql: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def skip_default(self, field: Any) -> Any: ...
    def column_sql(self, model: Any, field: Any, include_default: bool = ...) -> Any: ...
    def create_model(self, model: Any) -> None: ...
    def add_field(self, model: Any, field: Any) -> None: ...
    def remove_field(self, model: Any, field: Any) -> None: ...
    def create_spatial_indexes(self) -> None: ...



from typing import (
    Any,
    Collection,
    Dict,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    MutableSet,
    Optional,
    Tuple,
    TypeVar,
    Union,
    overload,
)

from typing_extensions import Protocol

_K = TypeVar("_K")
_V = TypeVar("_V")
_Z = TypeVar("_Z")
_I = TypeVar("_I", covariant=True)

# Unfortunately, there's often check `if isinstance(var, (list, tuple))` in django
# codebase. So we need sometimes to declare exactly list or tuple.
_ListOrTuple = Union[List[_K], Tuple[_K, ...], Tuple[()]]

class _PropertyDescriptor(Generic[_K, _V]):
    """
    This helper property descriptor allows defining asynmetric getter/setters
    which mypy currently doesn't support with either:

        class HttpResponse:
            @property
            def content(...): ...
            @property.setter
            def content(...): ...

    or:

        class HttpResponse:
            def _get_content(...): ...
            def _set_content(...): ...
            content = property(_get_content, _set_content)
    """

    def __get__(self, instance: Any, owner: Optional[Any]) -> _V: ...
    def __set__(self, instance: Any, value: _K) -> None: ...

_IC = TypeVar("_IC", bound="_IndexableCollection")

class _IndexableCollection(Protocol[_I], Collection[_I]):
    @overload
    def __getitem__(self, index: int) -> _I: ...
    @overload
    def __getitem__(self: _IC, index: slice) -> _IC: ...

class OrderedSet(MutableSet[_K]):
    dict: Dict[_K, None] = ...
    def __init__(self, iterable: Optional[Iterable[_K]] = ...) -> None: ...
    def __contains__(self, item: object) -> bool: ...
    def __iter__(self) -> Iterator[_K]: ...
    def __bool__(self) -> bool: ...
    def __len__(self) -> int: ...
    def add(self, item: _K) -> None: ...
    def remove(self, item: _K) -> None: ...
    def discard(self, item: _K) -> None: ...

class MultiValueDictKeyError(KeyError): ...

_D = TypeVar("_D", bound="MultiValueDict")

class MultiValueDict(Dict[_K, _V]):
    @overload
    def __init__(self, key_to_list_mapping: Mapping[_K, Optional[List[_V]]] = ...) -> None: ...
    @overload
    def __init__(self, key_to_list_mapping: Iterable[Tuple[_K, List[_V]]] = ...) -> None: ...
    @overload
    def get(self, key: _K) -> Optional[_V]: ...
    @overload
    def get(self, key: _K, default: _Z = ...) -> Union[_V, _Z]: ...
    def getlist(self, key: _K, default: _Z = ...) -> Union[List[_V], _Z]: ...
    def setlist(self, key: _K, list_: List[_V]) -> None: ...
    def setdefault(self, key: _K, default: _V = ...) -> _V: ...
    def setlistdefault(self, key: _K, default_list: Optional[List[_V]] = ...) -> List[_V]: ...
    def appendlist(self, key: _K, value: _V) -> None: ...
    def items(self) -> Iterator[Tuple[_K, Union[_V, List[object]]]]: ...  # type: ignore
    def lists(self) -> Iterable[Tuple[_K, List[_V]]]: ...
    def dict(self) -> Dict[_K, Union[_V, List[object]]]: ...
    def copy(self: _D) -> _D: ...
    def __getitem__(self, key: _K) -> Union[_V, List[object]]: ...  # type: ignore
    def __setitem__(self, key: _K, value: _V) -> None: ...
    # These overrides are needed to convince mypy that this isn't an abstract class
    def __delitem__(self, item: _K) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_K]: ...
    # Fake to make `values` work properly
    def values(self) -> Iterator[Union[_V, List[object]]]: ...  # type: ignore[override]

class ImmutableList(Tuple[_V, ...]):
    warning: str = ...
    def __new__(cls, *args: Any, warning: str = ..., **kwargs: Any) -> ImmutableList: ...
    def complain(self, *args: Any, **kwargs: Any) -> None: ...

class _ItemCallable(Protocol[_V]):
    """Don't mess with arguments when assigning in class body in stub"""

    def __call__(self, __value: _V) -> _V: ...

class DictWrapper(Dict[str, _V]):
    func: _ItemCallable[_V] = ...
    prefix: str = ...
    @overload
    def __init__(self, data: Mapping[str, _V], func: _ItemCallable[_V], prefix: str) -> None: ...
    @overload
    def __init__(self, data: Iterable[Tuple[str, _V]], func: _ItemCallable[_V], prefix: str) -> None: ...
    def __getitem__(self, key: str) -> _V: ...

_T = TypeVar("_T", bound="CaseInsensitiveMapping")

class CaseInsensitiveMapping(Mapping[str, _V]):
    _store: Dict[str, Tuple[str, _V]]
    def __init__(self, data: Union[Mapping[str, _V], Iterable[Tuple[str, _V]]]) -> None: ...
    def __getitem__(self, key: str) -> _V: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[str]: ...
    def copy(self: _T) -> _T: ...



from typing import Any, Callable, List, Optional, Sequence, Set, TypeVar, Union

from django.apps.config import AppConfig
from django.core.checks.messages import CheckMessage
from typing_extensions import Protocol

class Tags:
    admin: str = ...
    caches: str = ...
    compatibility: str = ...
    database: str = ...
    models: str = ...
    security: str = ...
    signals: str = ...
    sites: str = ...
    staticfiles: str = ...
    templates: str = ...
    translation: str = ...
    urls: str = ...

_CheckCallable = Callable[..., Sequence[CheckMessage]]
_C = TypeVar("_C", bound=_CheckCallable)

class _ProcessedCheckCallable(Protocol[_C]):
    tags: Sequence[str]
    __call__: _C

class CheckRegistry:
    registered_checks: Set[_ProcessedCheckCallable] = ...
    deployment_checks: Set[_ProcessedCheckCallable] = ...
    def __init__(self) -> None: ...
    def register(
        self, check: Optional[Union[_CheckCallable, str]] = ..., *tags: str, **kwargs: Any
    ) -> Union[Callable[[_CheckCallable], _ProcessedCheckCallable], _ProcessedCheckCallable]: ...
    def run_checks(
        self,
        app_configs: Optional[Sequence[AppConfig]] = ...,
        tags: Optional[Sequence[str]] = ...,
        include_deployment_checks: bool = ...,
        databases: Optional[Any] = ...,
    ) -> List[CheckMessage]: ...
    def tag_exists(self, tag: str, include_deployment_checks: bool = ...) -> bool: ...
    def tags_available(self, deployment_checks: bool = ...) -> Set[str]: ...
    def get_checks(self, include_deployment_checks: bool = ...) -> List[_ProcessedCheckCallable]: ...

registry: CheckRegistry = ...
register: Any = ...
run_checks: Any = ...
tag_exists: Any = ...



from typing import IO, Optional, Tuple, Union

from django.core.files import File
from django.utils._os import _PathCompatible

class ImageFile(File[bytes]):
    @property
    def width(self) -> int: ...
    @property
    def height(self) -> int: ...

def get_image_dimensions(
    file_or_path: Union[_PathCompatible, IO[bytes]], close: bool = ...
) -> Tuple[Optional[int], Optional[int]]: ...



from types import TracebackType
from typing import Any, Dict, Iterable, Optional, Sequence, Type

class ArchiveException(Exception): ...
class UnrecognizedArchiveFormat(ArchiveException): ...

def extract(path: str, to_path: str) -> None: ...

class Archive:
    def __init__(self, file: str) -> None: ...
    def __enter__(self) -> Archive: ...
    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> None: ...
    def extract(self, to_path: str) -> None: ...
    def list(self) -> None: ...
    def close(self) -> None: ...

class BaseArchive:
    def split_leading_dir(self, path: str) -> Sequence[str]: ...
    def has_leading_dir(self, paths: Iterable[str]) -> bool: ...
    def target_filename(self, to_path: str, name: str) -> str: ...
    def extract(self, to_path: str) -> None: ...
    def list(self, *args: Any, **kwargs: Any) -> None: ...

class TarArchive(BaseArchive):
    def __init__(self, file: str) -> None: ...
    def list(self, *args: Any, **kwargs: Any) -> None: ...
    def close(self) -> None: ...
    def extract(self, to_path: str) -> None: ...

class ZipArchive(BaseArchive):
    def __init__(self, file: str) -> None: ...
    def list(self, *args: Any, **kwargs: Any) -> None: ...
    def extract(self, to_path: str) -> None: ...
    def close(self) -> None: ...

extension_map: Dict[str, Type[BaseArchive]]



import threading
from typing import TextIO

from django.core.mail.backends.base import BaseEmailBackend

class EmailBackend(BaseEmailBackend):
    stream: TextIO = ...
    _lock: threading.RLock = ...



from typing import Any, Dict, Iterable, List, Tuple

from django.db.backends.base.client import BaseDatabaseClient as BaseDatabaseClient
from django.db.backends.sqlite3.base import DatabaseWrapper

class DatabaseClient(BaseDatabaseClient):
    connection: DatabaseWrapper
    executable_name: str = ...
    @classmethod
    def settings_to_cmd_args_env(
        cls, settings_dict: Dict[str, Any], parameters: Iterable[str]
    ) -> Tuple[List[str], None]: ...



from pathlib import Path
from typing import Iterator, List, Optional, Union

from django.template.base import Origin
from django.template.engine import Engine

from .base import Loader as BaseLoader

class Loader(BaseLoader):
    dirs: Optional[List[Union[str, Path]]] = ...
    def __init__(self, engine: Engine, dirs: Optional[List[Union[str, Path]]] = ...) -> None: ...
    def get_dirs(self) -> List[Union[str, Path]]: ...
    def get_contents(self, origin: Origin) -> str: ...
    def get_template_sources(self, template_name: str) -> Iterator[Origin]: ...



from typing import IO, Any, Iterator, Union

from django.core.serializers.base import DeserializedObject
from django.core.serializers.python import Serializer as PythonSerializer
from django.db.models.fields import Field
from yaml import CSafeDumper as SafeDumper
from yaml import MappingNode, ScalarNode

class DjangoSafeDumper(SafeDumper):
    def represent_decimal(self, data: Any) -> ScalarNode: ...
    def represent_ordered_dict(self, data: Any) -> MappingNode: ...

class Serializer(PythonSerializer):
    internal_use_only: bool = ...
    def handle_field(self, obj: Any, field: Field) -> None: ...
    def end_serialization(self) -> None: ...
    def getvalue(self) -> Any: ...

def Deserializer(
    stream_or_string: Union[bytes, str, IO[bytes], IO[str]], **options: Any
) -> Iterator[DeserializedObject]: ...



from typing import Set, Tuple

from django.db.migrations.serializer import BaseSerializer as BaseSerializer

class RangeSerializer(BaseSerializer):
    def serialize(self) -> Tuple[str, Set[str]]: ...



from typing import Any, Dict, List, Sequence, Tuple, Type, Union

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.migrations.state import ProjectState
from django.db.models import Model

class Operation:
    reversible: bool = ...
    reduces_to_sql: bool = ...
    atomic: bool = ...
    elidable: bool = ...
    serialization_expand_args: List[str] = ...
    _constructor_args: Tuple[Sequence[Any], Dict[str, Any]]
    def deconstruct(self) -> Tuple[str, Sequence[Any], Dict[str, Any]]: ...
    def state_forwards(self, app_label: str, state: ProjectState) -> None: ...
    def database_forwards(
        self, app_label: str, schema_editor: BaseDatabaseSchemaEditor, from_state: ProjectState, to_state: ProjectState
    ) -> None: ...
    def database_backwards(
        self, app_label: str, schema_editor: BaseDatabaseSchemaEditor, from_state: ProjectState, to_state: ProjectState
    ) -> None: ...
    def describe(self) -> str: ...
    def references_model(self, name: str, app_label: str) -> bool: ...
    def references_field(self, model_name: str, name: str, app_label: str) -> bool: ...
    def allow_migrate_model(self, connection_alias: Union[BaseDatabaseWrapper, str], model: Type[Model]) -> bool: ...
    def reduce(self, operation: Operation, app_label: str) -> Union[bool, List[Operation]]: ...
    @property
    def migration_name_fragment(self) -> str: ...



from typing import Any, Dict, List, Optional, Sequence, Tuple, Union

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models.expressions import Expression
from django.db.models.sql.compiler import SQLCompiler, _AsSqlType
from django.db.models.sql.query import Query
from django.utils import tree

AND: str
OR: str

class WhereNode(tree.Node):
    connector: str
    negated: bool
    default: str = ...
    resolved: bool = ...
    conditional: bool = ...
    def split_having(self, negated: bool = ...) -> Tuple[Optional[WhereNode], Optional[WhereNode]]: ...
    def as_sql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> Any: ...
    def get_group_by_cols(self, alias: Optional[str] = ...) -> List[Expression]: ...
    def relabel_aliases(self, change_map: Dict[Optional[str], str]) -> None: ...
    def clone(self) -> WhereNode: ...
    def relabeled_clone(self, change_map: Dict[Optional[str], str]) -> WhereNode: ...
    def resolve_expression(self, *args: Any, **kwargs: Any) -> WhereNode: ...
    @property
    def contains_aggregate(self) -> bool: ...
    @property
    def contains_over_clause(self) -> bool: ...
    @property
    def is_summary(self) -> bool: ...

class NothingNode:
    contains_aggregate: bool = ...
    def as_sql(
        self, compiler: Optional[SQLCompiler] = ..., connection: Optional[BaseDatabaseWrapper] = ...
    ) -> _AsSqlType: ...

class ExtraWhere:
    contains_aggregate: bool = ...
    sqls: Sequence[str] = ...
    params: Optional[Union[Sequence[int], Sequence[str]]] = ...
    def __init__(self, sqls: Sequence[str], params: Optional[Union[Sequence[int], Sequence[str]]]) -> None: ...
    def as_sql(
        self, compiler: Optional[SQLCompiler] = ..., connection: Optional[BaseDatabaseWrapper] = ...
    ) -> _AsSqlType: ...

class SubqueryConstraint:
    contains_aggregate: bool = ...
    alias: str = ...
    columns: List[str] = ...
    targets: List[str] = ...
    query_object: Query = ...
    def __init__(self, alias: str, columns: List[str], targets: List[str], query_object: Query) -> None: ...
    def as_sql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...



import json
from typing import IO, Any, Dict, Iterator, Union

from django.core.serializers.base import DeserializedObject
from django.core.serializers.python import Serializer as PythonSerializer

class Serializer(PythonSerializer):
    json_kwargs: Dict[str, Any]

def Deserializer(
    stream_or_string: Union[IO[bytes], IO[str], bytes, str], **options: Any
) -> Iterator[DeserializedObject]: ...

class DjangoJSONEncoder(json.JSONEncoder):
    allow_nan: bool
    check_circular: bool
    ensure_ascii: bool
    indent: int
    skipkeys: bool
    sort_keys: bool



import types
from datetime import date
from datetime import datetime as builtin_datetime
from datetime import time
from decimal import Decimal
from typing import Any, Dict, FrozenSet, Iterator, List, Optional, TypeVar, Union, overload

ISO_INPUT_FORMATS: Dict[str, List[str]]
FORMAT_SETTINGS: FrozenSet[str]

def reset_format_cache() -> None: ...
def iter_format_modules(
    lang: str, format_module_path: Optional[Union[List[str], str]] = ...
) -> Iterator[types.ModuleType]: ...
def get_format_modules(lang: Optional[str] = ..., reverse: bool = ...) -> List[types.ModuleType]: ...
def get_format(format_type: str, lang: Optional[str] = ..., use_l10n: Optional[bool] = ...) -> Any: ...

get_format_lazy: Any

def date_format(
    value: Union[date, builtin_datetime, str], format: Optional[str] = ..., use_l10n: Optional[bool] = ...
) -> str: ...
def time_format(
    value: Union[time, builtin_datetime, str], format: Optional[str] = ..., use_l10n: Optional[bool] = ...
) -> str: ...
def number_format(
    value: Union[Decimal, float, str],
    decimal_pos: Optional[int] = ...,
    use_l10n: Optional[bool] = ...,
    force_grouping: bool = ...,
) -> str: ...

_T = TypeVar("_T")

# Mypy considers this invalid (overlapping signatures), but thanks to implementation
# details it works as expected (all values from Union are `localize`d to str,
# while type of others is preserved)
@overload
def localize(  # type: ignore
    value: Union[builtin_datetime, date, time, Decimal, float, str], use_l10n: Optional[bool] = ...
) -> str: ...
@overload
def localize(value: _T, use_l10n: Optional[bool] = ...) -> _T: ...
@overload
def localize_input(  # type: ignore
    value: Union[builtin_datetime, date, time, Decimal, float, str], default: Optional[str] = ...
) -> str: ...
@overload
def localize_input(value: _T, default: Optional[str] = ...) -> _T: ...
def sanitize_separators(value: _T) -> _T: ...



from typing import Any, Callable, Iterable, List, Optional, Pattern, Tuple, Union

from django.db.models.fields import Field
from django.urls import _AnyURL
from django.views.generic import TemplateView

MODEL_METHODS_EXCLUDE: Any

class BaseAdminDocsView(TemplateView): ...
class BookmarkletsView(BaseAdminDocsView): ...
class TemplateTagIndexView(BaseAdminDocsView): ...
class TemplateFilterIndexView(BaseAdminDocsView): ...
class ViewIndexView(BaseAdminDocsView): ...
class ViewDetailView(BaseAdminDocsView): ...
class ModelIndexView(BaseAdminDocsView): ...
class ModelDetailView(BaseAdminDocsView): ...
class TemplateDetailView(BaseAdminDocsView): ...

def get_return_data_type(func_name: Any) -> str: ...
def get_readable_field_data_type(field: Union[Field, str]) -> str: ...
def extract_views_from_urlpatterns(
    urlpatterns: Iterable[_AnyURL], base: str = ..., namespace: Optional[str] = ...
) -> List[Tuple[Callable, Pattern[str], Optional[str], Optional[str]]]: ...
def simplify_regex(pattern: str) -> str: ...



from typing import Any, Dict, Optional, TypeVar, Union

from django.db.models import Expression, Field
from django.db.models.expressions import Combinable, CombinedExpression, Func, Value
from django.db.models.lookups import Lookup

_Expression = Union[str, Combinable, "SearchQueryCombinable"]

class SearchVectorExact(Lookup): ...
class SearchVectorField(Field): ...
class SearchQueryField(Field): ...

class SearchConfig(Expression):
    config: Optional[_Expression] = ...
    def __init__(self, config: _Expression) -> None: ...
    @classmethod
    def from_parameter(cls, config: Optional[_Expression]) -> SearchConfig: ...

class SearchVectorCombinable:
    ADD: str = ...

class SearchVector(SearchVectorCombinable, Func):
    config: Optional[_Expression] = ...
    function: str = ...
    arg_joiner: str = ...
    output_field: Field
    def __init__(
        self, *expressions: _Expression, config: Optional[_Expression] = ..., weight: Optional[Any] = ...
    ) -> None: ...

class CombinedSearchVector(SearchVectorCombinable, CombinedExpression):
    def __init__(
        self,
        lhs: Combinable,
        connector: str,
        rhs: Combinable,
        config: Optional[_Expression],
        output_field: Optional[Field] = None,
    ) -> None: ...

_T = TypeVar("_T", bound="SearchQueryCombinable")

class SearchQueryCombinable:
    BITAND: str = ...
    BITOR: str = ...
    def __or__(self: _T, other: SearchQueryCombinable) -> _T: ...
    def __ror__(self: _T, other: SearchQueryCombinable) -> _T: ...
    def __and__(self: _T, other: SearchQueryCombinable) -> _T: ...
    def __rand__(self: _T, other: SearchQueryCombinable) -> _T: ...

class SearchQuery(SearchQueryCombinable, Func):  # type: ignore
    SEARCH_TYPES: Dict[str, str] = ...
    def __init__(
        self,
        value: _Expression,
        output_field: Optional[Field] = ...,
        *,
        config: Optional[_Expression] = ...,
        invert: bool = ...,
        search_type: str = ...,
    ) -> None: ...
    def __invert__(self: _T) -> _T: ...

class CombinedSearchQuery(SearchQueryCombinable, CombinedExpression):  # type: ignore
    def __init__(
        self,
        lhs: Combinable,
        connector: str,
        rhs: Combinable,
        config: Optional[_Expression],
        output_field: Optional[Field] = None,
    ) -> None: ...

class SearchRank(Func):
    def __init__(
        self,
        vector: Union[SearchVector, _Expression],
        query: Union[SearchQuery, _Expression],
        weights: Optional[Any] = ...,
        normalization: Optional[Any] = ...,
        cover_density: bool = ...,
    ) -> None: ...

class SearchHeadline(Func):
    function: str = ...
    template: str = ...
    output_field: Field = ...
    def __init__(
        self,
        expression: _Expression,
        query: _Expression,
        *,
        config: Optional[_Expression] = ...,
        start_sel: Optional[Any] = ...,
        stop_sel: Optional[Any] = ...,
        max_words: Optional[int] = ...,
        min_words: Optional[int] = ...,
        short_word: Optional[str] = ...,
        highlight_all: Optional[bool] = ...,
        max_fragments: Optional[int] = ...,
        fragment_delimiter: Optional[str] = ...,
    ) -> None: ...

class TrigramBase(Func):
    def __init__(self, expression: _Expression, string: str, **extra: Any) -> None: ...

class TrigramSimilarity(TrigramBase):
    function: str

class TrigramDistance(TrigramBase):
    function: str
    arg_joiner: str



import datetime
from decimal import Decimal
from typing import Any, Callable, Dict, Iterable, Iterator, List, Optional, Sequence, Set, Tuple, Type, TypeVar, Union

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models import Q
from django.db.models.fields import Field
from django.db.models.lookups import Lookup, Transform
from django.db.models.query import QuerySet
from django.db.models.sql.compiler import SQLCompiler, _AsSqlType
from django.db.models.sql.query import Query
from typing_extensions import Literal

class SQLiteNumericMixin:
    def as_sqlite(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...

_Self = TypeVar("_Self")
_Numeric = Union[float, Decimal]

class Combinable:
    ADD: str = ...
    SUB: str = ...
    MUL: str = ...
    DIV: str = ...
    POW: str = ...
    MOD: str = ...
    BITAND: str = ...
    BITOR: str = ...
    BITLEFTSHIFT: str = ...
    BITRIGHTSHIFT: str = ...
    BITXOR: str = ...
    def __neg__(self) -> CombinedExpression: ...
    def __add__(self, other: Optional[Union[datetime.timedelta, Combinable, _Numeric, str]]) -> CombinedExpression: ...
    def __sub__(self, other: Union[datetime.timedelta, Combinable, _Numeric]) -> CombinedExpression: ...
    def __mul__(self, other: Union[datetime.timedelta, Combinable, _Numeric]) -> CombinedExpression: ...
    def __truediv__(self, other: Union[Combinable, _Numeric]) -> CombinedExpression: ...
    def __mod__(self, other: Union[int, Combinable]) -> CombinedExpression: ...
    def __pow__(self, other: Union[_Numeric, Combinable]) -> CombinedExpression: ...
    def __and__(self, other: Union[Combinable, Q]) -> Q: ...
    def bitand(self, other: int) -> CombinedExpression: ...
    def bitleftshift(self, other: int) -> CombinedExpression: ...
    def bitrightshift(self, other: int) -> CombinedExpression: ...
    def bitxor(self, other: int) -> CombinedExpression: ...
    def __or__(self, other: Union[Combinable, Q]) -> Q: ...
    def bitor(self, other: int) -> CombinedExpression: ...
    def __radd__(self, other: Optional[Union[datetime.datetime, _Numeric, Combinable]]) -> CombinedExpression: ...
    def __rsub__(self, other: Union[_Numeric, Combinable]) -> CombinedExpression: ...
    def __rmul__(self, other: Union[_Numeric, Combinable]) -> CombinedExpression: ...
    def __rtruediv__(self, other: Union[_Numeric, Combinable]) -> CombinedExpression: ...
    def __rmod__(self, other: Union[int, Combinable]) -> CombinedExpression: ...
    def __rpow__(self, other: Union[_Numeric, Combinable]) -> CombinedExpression: ...
    def __rand__(self, other: Any) -> Combinable: ...
    def __ror__(self, other: Any) -> Combinable: ...

_SelfB = TypeVar("_SelfB", bound="BaseExpression")

class BaseExpression:
    is_summary: bool = ...
    filterable: bool = ...
    window_compatible: bool = ...
    def __init__(self, output_field: Optional[Field] = ...) -> None: ...
    def get_db_converters(self, connection: BaseDatabaseWrapper) -> List[Callable]: ...
    def get_source_expressions(self) -> List[Any]: ...
    def set_source_expressions(self, exprs: Sequence[Combinable]) -> None: ...
    @property
    def contains_aggregate(self) -> bool: ...
    @property
    def contains_over_clause(self) -> bool: ...
    @property
    def contains_column_references(self) -> bool: ...
    def resolve_expression(
        self: _SelfB,
        query: Any = ...,
        allow_joins: bool = ...,
        reuse: Optional[Set[str]] = ...,
        summarize: bool = ...,
        for_save: bool = ...,
    ) -> _SelfB: ...
    @property
    def conditional(self) -> bool: ...
    @property
    def field(self) -> Field: ...
    @property
    def output_field(self) -> Field: ...
    @property
    def convert_value(self) -> Callable: ...
    def get_lookup(self, lookup: str) -> Optional[Type[Lookup]]: ...
    def get_transform(self, name: str) -> Optional[Type[Transform]]: ...
    def relabeled_clone(self: _SelfB, change_map: Dict[Optional[str], str]) -> _SelfB: ...
    def copy(self: _SelfB) -> _SelfB: ...
    def get_group_by_cols(self: _SelfB, alias: Optional[str] = ...) -> List[_SelfB]: ...
    def get_source_fields(self) -> List[Optional[Field]]: ...
    def asc(
        self,
        *,
        descending: bool = ...,
        nulls_first: bool = ...,
        nulls_last: bool = ...,
    ) -> OrderBy: ...
    def desc(
        self,
        *,
        nulls_first: bool = ...,
        nulls_last: bool = ...,
    ) -> OrderBy: ...
    def reverse_ordering(self) -> BaseExpression: ...
    def flatten(self) -> Iterator[BaseExpression]: ...
    def as_sql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    def deconstruct(self) -> Any: ...  # fake

class Expression(BaseExpression, Combinable): ...

class CombinedExpression(SQLiteNumericMixin, Expression):
    connector: str = ...
    lhs: Combinable = ...
    rhs: Combinable = ...
    def __init__(
        self, lhs: Combinable, connector: str, rhs: Combinable, output_field: Optional[Field] = ...
    ) -> None: ...

class F(Combinable):
    name: str
    def __init__(self, name: str) -> None: ...
    def resolve_expression(
        self,
        query: Any = ...,
        allow_joins: bool = ...,
        reuse: Optional[Set[str]] = ...,
        summarize: bool = ...,
        for_save: bool = ...,
    ) -> F: ...
    def asc(
        self,
        *,
        descending: bool = ...,
        nulls_first: bool = ...,
        nulls_last: bool = ...,
    ) -> OrderBy: ...
    def desc(
        self,
        *,
        nulls_first: bool = ...,
        nulls_last: bool = ...,
    ) -> OrderBy: ...
    def deconstruct(self) -> Any: ...  # fake

class ResolvedOuterRef(F): ...

class OuterRef(F):
    def __init__(self, name: Union[str, OuterRef]) -> None: ...
    contains_aggregate: bool
    def relabeled_clone(self: _Self, relabels: Any) -> _Self: ...

class Subquery(BaseExpression, Combinable):
    template: str = ...
    query: Query = ...
    extra: Dict[Any, Any] = ...
    def __init__(self, queryset: Union[Query, QuerySet], output_field: Optional[Field] = ..., **extra: Any) -> None: ...

class Exists(Subquery):
    negated: bool = ...
    def __init__(self, queryset: Union[Query, QuerySet], negated: bool = ..., **kwargs: Any) -> None: ...
    def __invert__(self) -> Exists: ...

class OrderBy(Expression):
    template: str = ...
    nulls_first: bool = ...
    nulls_last: bool = ...
    descending: bool = ...
    expression: Union[Expression, F, Subquery] = ...
    def __init__(
        self,
        expression: Union[Expression, F, Subquery],
        descending: bool = ...,
        nulls_first: bool = ...,
        nulls_last: bool = ...,
    ) -> None: ...

class Value(Expression):
    value: Any = ...
    def __init__(self, value: Any, output_field: Optional[Field] = ...) -> None: ...

class RawSQL(Expression):
    params: List[Any]
    sql: str
    def __init__(self, sql: str, params: Sequence[Any], output_field: Optional[Field] = ...) -> None: ...

class Func(SQLiteNumericMixin, Expression):
    function: str = ...
    name: str = ...
    template: str = ...
    arg_joiner: str = ...
    arity: Optional[int] = ...
    source_expressions: List[Expression] = ...
    extra: Dict[Any, Any] = ...
    def __init__(self, *expressions: Any, output_field: Optional[Field] = ..., **extra: Any) -> None: ...

class When(Expression):
    template: str = ...
    condition: Any = ...
    result: Any = ...
    def __init__(self, condition: Any = ..., then: Any = ..., **lookups: Any) -> None: ...

class Case(Expression):
    template: str = ...
    case_joiner: str = ...
    cases: Any = ...
    default: Any = ...
    extra: Any = ...
    def __init__(
        self, *cases: Any, default: Optional[Any] = ..., output_field: Optional[Field] = ..., **extra: Any
    ) -> None: ...

class ExpressionWrapper(Expression):
    def __init__(self, expression: Union[Q, Combinable], output_field: Field) -> None: ...

class Col(Expression):
    target: Field
    alias: str
    contains_column_references: Literal[True] = ...
    possibly_multivalued: Literal[False] = ...
    def __init__(self, alias: str, target: Field, output_field: Optional[Field] = ...) -> None: ...

class Ref(Expression):
    def __init__(self, refs: str, source: Expression) -> None: ...

class ExpressionList(Func):
    def __init__(self, *expressions: Union[BaseExpression, Combinable], **extra: Any) -> None: ...

class Window(SQLiteNumericMixin, Expression):
    template: str = ...
    contains_aggregate: bool = ...
    contains_over_clause: bool = ...
    partition_by: Optional[ExpressionList]
    order_by: Optional[ExpressionList]
    def __init__(
        self,
        expression: BaseExpression,
        partition_by: Optional[Union[str, Iterable[Union[BaseExpression, F]], F, BaseExpression]] = ...,
        order_by: Optional[Union[Sequence[Union[BaseExpression, F]], Union[BaseExpression, F]]] = ...,
        frame: Optional[WindowFrame] = ...,
        output_field: Optional[Field] = ...,
    ) -> None: ...

class WindowFrame(Expression):
    template: str = ...
    frame_type: str = ...
    def __init__(self, start: Optional[int] = ..., end: Optional[int] = ...) -> None: ...
    def window_frame_start_end(
        self, connection: BaseDatabaseWrapper, start: Optional[int], end: Optional[int]
    ) -> Tuple[int, int]: ...

class RowRange(WindowFrame): ...
class ValueRange(WindowFrame): ...



from typing import Any, Mapping, Optional, Sequence, Union

from django.http.request import HttpRequest
from django.template.exceptions import TemplateDoesNotExist as TemplateDoesNotExist  # noqa: F401
from django.utils.safestring import SafeString

from . import engines as engines  # noqa: F401
from .backends.base import _EngineTemplate

def get_template(template_name: str, using: Optional[str] = ...) -> _EngineTemplate: ...
def select_template(template_name_list: Union[Sequence[str], str], using: Optional[str] = ...) -> Any: ...
def render_to_string(
    template_name: Union[Sequence[str], str],
    context: Optional[Mapping[str, Any]] = ...,
    request: Optional[HttpRequest] = ...,
    using: Optional[str] = ...,
) -> SafeString: ...



from typing import Any, Callable, Dict, List, Optional, Tuple

from django.utils.safestring import SafeString

docutils_is_available: bool

def get_view_name(view_func: Callable) -> str: ...
def parse_docstring(docstring: str) -> Tuple[str, str, Dict[str, str]]: ...
def parse_rst(text: str, default_reference_context: Any, thing_being_parsed: Optional[Any] = ...) -> SafeString: ...

ROLES: Dict[str, str]

def create_reference_role(rolename: str, urlbase: str) -> None: ...
def default_reference_role(
    name: str,
    rawtext: str,
    text: str,
    lineno: Any,
    inliner: Any,
    options: Optional[Any] = ...,
    content: Optional[Any] = ...,
) -> Tuple[List[Any], List[Any]]: ...

named_group_matcher: Any
unnamed_group_matcher: Any

def replace_named_groups(pattern: str) -> str: ...
def replace_unnamed_groups(pattern: str) -> str: ...



from typing import Any, Optional, Sequence

from django.apps.config import AppConfig
from django.core.checks.messages import Error, Warning

E001: Error

def check_default_cache_is_configured(app_configs: Optional[Sequence[AppConfig]], **kwargs: Any) -> Sequence[Error]: ...
def check_cache_location_not_exposed(
    app_configs: Optional[Sequence[AppConfig]], **kwargs: Any
) -> Sequence[Warning]: ...
def check_file_based_cache_is_absolute(
    app_configs: Optional[Sequence[AppConfig]], **kwargs: Any
) -> Sequence[Warning]: ...



from typing import Dict, Pattern, Tuple

from typing_extensions import Final, Literal

GET_ITERATOR_CHUNK_SIZE: Final[int] = ...

MULTI: Literal["multi"] = ...
SINGLE: Literal["single"] = ...
CURSOR: Literal["cursor"] = ...
NO_RESULTS: Literal["no results"] = ...

ORDER_PATTERN: Pattern = ...
ORDER_DIR: Dict[str, Tuple[str, str]] = ...

INNER: Literal["INNER JOIN"] = ...
LOUTER: Literal["LEFT OUTER JOIN"] = ...



from collections import namedtuple
from typing import (
    Any,
    Collection,
    Dict,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models.base import Model
from django.db.models.expressions import BaseExpression
from django.db.models.fields import Field
from django.db.models.fields.mixins import FieldCacheMixin
from django.db.models.lookups import Lookup, Transform
from django.db.models.sql.compiler import SQLCompiler, _AsSqlType
from django.db.models.sql.query import Query
from django.db.models.sql.where import WhereNode
from django.utils import tree
from typing_extensions import Literal

PathInfo = namedtuple(
    "PathInfo", ["from_opts", "to_opts", "target_fields", "join_field", "m2m", "direct", "filtered_relation"]
)

class InvalidQuery(Exception): ...

def subclasses(cls: Type[RegisterLookupMixin]) -> Iterator[Type[RegisterLookupMixin]]: ...

class Q(tree.Node):
    AND: str = ...
    OR: str = ...
    conditional: bool = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    # Fake signature, the real is
    # def __init__(self, *args: Any, _connector: Optional[Any] = ..., _negated: bool = ..., **kwargs: Any) -> None: ...
    def __or__(self, other: Q) -> Q: ...
    def __and__(self, other: Q) -> Q: ...
    def __xor__(self, other: Q) -> Q: ...
    def __invert__(self) -> Q: ...
    def resolve_expression(
        self,
        query: Query = ...,
        allow_joins: bool = ...,
        reuse: Optional[Set[str]] = ...,
        summarize: bool = ...,
        for_save: bool = ...,
    ) -> WhereNode: ...
    def deconstruct(self) -> Tuple[str, Tuple, Dict[str, str]]: ...

class DeferredAttribute:
    field_name: str = ...
    field: Field
    def __init__(self, field: Field) -> None: ...

_R = TypeVar("_R", bound=Type)

class RegisterLookupMixin:
    class_lookups: List[Dict[Any, Any]]
    lookup_name: str
    @classmethod
    def get_lookups(cls) -> Dict[str, Any]: ...
    def get_lookup(self, lookup_name: str) -> Optional[Type[Lookup]]: ...
    def get_transform(self, lookup_name: str) -> Optional[Type[Transform]]: ...
    @staticmethod
    def merge_dicts(dicts: Iterable[Dict[str, Any]]) -> Dict[str, Any]: ...
    @classmethod
    def register_lookup(cls, lookup: _R, lookup_name: Optional[str] = ...) -> _R: ...
    @classmethod
    def _unregister_lookup(cls, lookup: Type[Lookup], lookup_name: Optional[str] = ...) -> None: ...

def select_related_descend(
    field: Field,
    restricted: bool,
    requested: Optional[Mapping[str, Any]],
    load_fields: Optional[Collection[str]],
    reverse: bool = ...,
) -> bool: ...

_E = TypeVar("_E", bound=BaseExpression)

def refs_expression(
    lookup_parts: Sequence[str], annotations: Mapping[str, _E]
) -> Tuple[Union[Literal[False], _E], Sequence[str]]: ...
def check_rel_lookup_compatibility(model: Type[Model], target_opts: Any, field: FieldCacheMixin) -> bool: ...

class FilteredRelation:
    relation_name: str = ...
    alias: Optional[str] = ...
    condition: Q = ...
    path: List[str] = ...
    def __init__(self, relation_name: str, *, condition: Q = ...) -> None: ...
    def clone(self) -> FilteredRelation: ...
    def resolve_expression(self, *args: Any, **kwargs: Any) -> None: ...
    def as_sql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
